function [] = cal_agreed_label(src_list_file,pcf,maxyth,maxsth,dispflag)
% function [] = cal_agreed_label(src_list_file,pcf,maxyth,maxsth,dispflag)
%
% Calculate ground truth label of lists audio using Wu, YIN, SWIPE' agreement
% and save it to gth files
%
% [2012-07-20] Byung Suk Lee bsl@ee.columbia.edu

% Set the path
if nargin < 3, maxyth = 0.5900; end
if nargin < 4, maxsth = 0.1200; end
if nargin < 5, dispflag = 0; end

%% Make ground truth label for all source file list
algs = {'wu','yin','swipep'};
nalgs = length(algs);
fid = fopen(src_list_file,'r');
while 1
    line = fgetl(fid);
    if ~ischar(line), break, end
    files = regexp(line,',','split');
    disp(['Processing Input: ',files{1},' Output: ',files{2}])
    [out_path] = fileparts(files{2});
    yin_pfn = strrep(strrep(files{2},'.gth','.pitch'),'gth','yin');
    [ypath,dum_name,dum_exp] = fileparts(yin_pfn);
    wu_pfn = strrep(strrep(files{2},'.gth','.pitch'),'gth','wu');
    [wpath,dum_name,dum_exp] = fileparts(wu_pfn);
    swipep_pfn = strrep(strrep(files{2},'.gth','.pitch'),'gth','swipep');
    [spath,dum_name,dum_exp] = fileparts(swipep_pfn);
    if ~exist(out_path,'dir'), mkdir(out_path); end;
    if ~exist(ypath,'dir'), mkdir(ypath); end;
    if ~exist(wpath,'dir'), mkdir(wpath); end;
    if ~exist(spath,'dir'), mkdir(spath); end;
    maxl = 0;

    if ~exist(files{2},'file')
    if ~exist(yin_pfn,'file') || ~exist(wu_pfn,'file') || ~exist(swipep_pfn,'file')
        [yf0,ap0,wf0,sf0,ss] = run_yws_slice(files{1});
        dlmwrite(yin_pfn,[yf0' ap0'],'delimiter','\t');
        dlmwrite(wu_pfn,wf0','delimiter','\t');
        dlmwrite(swipep_pfn,[sf0' ss'],'delimiter','\t');
    else
        yf0 = dlmread(yin_pfn);
        ap0 = yf0(:,2); yf0 = yf0(:,1);
        sf0 = dlmread(swipep_pfn);
        ss = sf0(:,2); sf0 = sf0(:,1);
        wf0 = dlmread(wu_pfn);
    end
    yf0(ap0 > maxyth) = 0;
    sf0(ss < maxsth) = 0;
    f0s{1} = wf0; f0s{2} = yf0; f0s{3} = sf0;
    for alg = 1:nalg
        maxl = max(maxl,length(f0s{alg}));
    end
    for alg = 1:nalg
        if length(f0s{alg}) < maxl
            f0s{alg}(end+1:maxl) = f0s{alg}(end);
        end
    end
    [pitch,sad,dc_idx,pf0] = genPitchLabel(f0s,algs,pcf,dispflag);
    dlmwrite(files{2},pitch')
    end
    
end
fclose(fid);

