function [P] = load_config(config_file)
% function [P] = load_config(config_file)
%
% Load config file and save the parameters in P
%
%% [2011-12-15] Byung Suk Lee bsl@ee.columbia.edu

fid = fopen(config_file,'r');
while 1
    line = fgetl(fid);
    if ~ischar(line), break, end
    pat = '=';
    s = regexp(line, pat, 'split');
    disp([s{1},'=',s{2}])
    val = str2double(s{2});
    if isnan(val)
        P.(genvarname(s{1})) = s{2};
    else
        P.(genvarname(s{1})) = val;
    end
end
fclose(fid);

%%
allbase = '../../';
if ~exist('P','var'), P = struct(); end;
if ~isfield(P,'alg'), P.alg = 'SAcC'; end;
if ~isfield(P,'dataset'), P.dataset= 'keele'; end;
if ~isfield(P,'ntype'), P.ntype = 'rbf_pinknoise'; end;
if ~isfield(P,'tst_dataset'), P.tst_dataset= 'fda'; end;
if ~isfield(P,'tst_ntype'), P.tst_ntype = 'rbf_pinknoise'; end;
if ~isfield(P,'kdim'), P.kdim = 10; end;
if ~isfield(P,'nchs'), P.nchs = 48; end;
if ~isfield(P,'fea_name'), P.fea_name = ['_sr',num2str(P.sr),'_bpo',num2str(P.bpo),'_nchs',num2str(P.nchs),'_k',num2str(P.kdim)]; end;
if ~isfield(P,'train_file'), P.train_file = ['tmp/tr_',P.dataset,'_',P.ntype,P.fea_name,'.pfile']; end;
if ~isfield(P,'train_asc'), P.train_asc = ['tmp/tr_',P.dataset,'_',P.ntype,P.fea_name,'.ascii']; end;
if ~isfield(P,'train_label'), P.train_label = ['tmp/tr_',P.dataset,'_',P.ntype,P.fea_name,'.lab.pfile']; end;
if ~isfield(P,'train_lasc'), P.train_lasc = ['tmp/tr_',P.dataset,'_',P.ntype,P.fea_name,'.lab.ascii']; end;
if ~isfield(P,'test_file'), P.test_file = ['tmp/tst_',P.tst_dataset,'_',P.tst_ntype,P.fea_name,'.pfile']; end;
if ~isfield(P,'test_asc'), P.test_asc = ['tmp/tst_',P.tst_dataset,'_',P.tst_ntype,P.fea_name,'.ascii']; end;
if ~isfield(P,'test_label'), P.test_label = ['tmp/tst_',P.tst_dataset,'_',P.tst_ntype,P.fea_name,'.lab.pfile']; end;
if ~isfield(P,'test_lasc'), P.test_lasc = ['tmp/tst_',P.tst_dataset,'_',P.tst_ntype,P.fea_name,'.lab.ascii']; end;
if ~isfield(P,'trR'), P.trR = 0.8; end; % QuickNet training sentences ratio (the rest is used for CV.)
if ~isfield(P,'hid'), P.hid = 50; end; % QuickNet hidden layer size
% if ~isfield(P,'hids'), P.hids = '50 100 200'; end; % QuickNet hidden layer size
if ~isfield(P,'wl'), P.wl = 1; end; % QuickNet context window length
if ~isfield(P,'QN_Learn_Rate_Param'), P.QN_Learn_Rate_Param = 0; end; % QN_Learn_Rate_Param
if ~isfield(P,'qe'), P.qe = 0; end;
if ~isfield(P,'q'), P.q = 8; end;
if ~isfield(P,'dispflag'), P.dispflag = 0; end;
%% :::START::: for HMM tracking
if ~isfield(P,'TransFloor'), P.TransFloor = -30; end;
if ~isfield(P,'wdy'), P.wdy = 1; end;
if ~isfield(P,'wob'), P.wob = 1; end;
%% ::: END ::: for HMM tracking
if ~isfield(P,'cleanpath'), P.cleanpath = [allbase,'dat/',P.dataset,'/']; end;
if ~isfield(P,'outpath'), P.outpath = [P.cleanpath,P.alg,'/out/']; end;
if ~isfield(P,'imgpath'), P.imgpath = [P.cleanpath,P.alg,'/images/']; end;
if ~exist(P.outpath,'dir'), mkdir(P.outpath); end;
if ~exist(P.imgpath,'dir'), mkdir(P.imgpath); end;

%% Get 20 candidate pitch lags per octave from 40Hz to around 4kHz
fmin = 60; nPitchPerOctave = 24; fmax = 400;
tmax = round(nPitchPerOctave*log2(fmax/fmin))/nPitchPerOctave;
t = 0:(1/nPitchPerOctave):tmax; % 154 pitch candidates to cover
P.pcf = fmin*2.^t; % center frequencies from 40Hz (400 lags) to around 8kHz
dt = 1/2/nPitchPerOctave + t; % the boundary of the subbands in the log-scale
P.pbf = fmin*2.^dt; % the boundary of the subbands
P.npcf = length(P.pcf);
P.nmlp = P.npcf+1;
P.force_mono = 1;
%%
P.maxlags = P.sr*0.025;
P.lags = 1:P.maxlags;

%% Design Lyon-Slaney filters
% fmin = 100; bpo = 16; q = 8; n = 2; ftype = 2; sr = 16000;
if P.qe == 0
[P.b2,P.a2,P.t2,P.w2,P.cf] = bpfiltbank(P.sr,P.fmin,P.bpo,P.nchs,P.q,P.n,P.ftype);  % type 2 is LySla
else
[P.b2,P.a2,P.t2,P.w2,P.cf] = bpfiltbank(P.sr,P.fmin,P.bpo,P.nchs,P.q,P.n,P.ftype,P.qe);  % To modify EarQ using EarQsqrt2e parameter 
end

%% load SBPCA features
load(P.pca_file,'coeff')
P.mapping = cell(P.nchs,1);
for sb = 1:P.nchs
    P.mapping{sb} = coeff{sb}(:,1:P.kdim);
end

%% QuickNet norm and weight files
[tmpdir,tmpfn,extdum] = fileparts(P.train_file);
P.train_norms = [tmpdir,'/',tmpfn,'.norms'];
[tmpdir,tmpfn,extdum] = fileparts(P.test_file);
P.test_norms = [tmpdir,'/',tmpfn,'.norms'];
P.tr_base = ['tmp/tr_',P.dataset,'_',P.ntype,P.fea_name];
P.tst_base = [P.tr_base,'/tst_',P.tst_dataset,'_',P.tst_ntype];
