function [] = qnstrn_wrap(f_fn,n_fn,l_fn,w_fn,w_log,w_chklog,w_chk,Fd,Ud,allR,trR,hid,wl,nMLP,QNLRP,f1s,f1c,reject)
% function [] = qnstrn_wrap(f_fn,n_fn,l_fn,w_fn,w_log,w_chklog,w_chk,Fd,Ud,allR,trR,hid,wl,nMLP,QNLRP,f1s,f1c,reject)
%
%    Matlab wrapper for QuickNet qnstrn for single label target
%
%    Input:
%           f_fn: feature pfile name
%           n_fn: feature norms pfile name
%           l_fn: label pfile name
%           w_fn: weight file name
%           w_log: log file name
%           w_chklog: weight log file name
%           w_chk: weight check file name
%           Fd: feature dimension
%           Ud: number of sentences
%           allR: Range of training+CV of sentences (strings) (default: 0:(Ud-1))
%           trR: Range of training of sentences (strings) (the rest is used in CV.)
%           hid: number of hidden layers
%           wl: window length
%           nMLP: number of MLP outputs (for SAcC, usually npcf + 1)
%           QNLRP: QN_Learn_Rate_Param (exponent of 10) (default: 0)
%           f1s: ftr1_ftr_start (default: 0)
%           f1c: ftr1_ftr_count (default: Fd)
%           reject: lastlab_reject enable flag (default: false)
%
%    Output:
%           None
%
% Wrapper for Quicknet training.
%% [2011-11-07] Byung Suk Lee bsl@ee.columbia.edu
%% Last Modified: [2012-05-09] Byung Suk Lee bsl@ee.columbia.edu

%% to support sub-select features
if (~exist('QNLRP','var') || isempty(QNLRP)), QNLRP = 0; end;
if (~exist('f1s','var') || isempty(f1s)), f1s = 0; end;
if (~exist('f1c','var') || isempty(f1c)), f1c = Fd; end;
if (~exist('allR','var') || isempty(allR)), allR = ['0:',num2str(Ud-1)]; end;
if (~exist('reject','var') || isempty(reject)), reject = 0; end;

qntrn = 'qnstrn';
trCV = [allR,'/',trR];
LRval = 0.008 * 10^(QNLRP);

%% run QuickNet
insize = f1c * wl;
woffset = (wl > 1) * round((wl - 1) / 2);
qo1 = [' ftr1_file=',f_fn,' hardtarget_file=',l_fn];
qo2 = [' ftr1_norm_file=',n_fn];
qo3 = [' ftr1_ftr_start=',num2str(f1s)];
qo4 = [' ftr1_ftr_count=',num2str(f1c)];
qo5 = [' window_extent=',num2str(wl + woffset),' mlp3_input_size=',num2str(insize)];
qo6 = [' mlp3_hidden_size=',num2str(hid),' mlp3_output_size=',num2str(nMLP)];
qo7 = [' train_sent_range=',trR,' train_cache_frames=','100000'];
qo8 = [' cv_sent_range=',trCV,' learnrate_vals=',num2str(LRval)];
qo9 = [' ftr1_window_len=',num2str(wl)];
qo9a = [' ftr1_window_offset=',num2str(woffset)];]
qo10 = [' ftr1_delta_order=','0'];
qo11 = [' ftr1_delta_win=','1',' out_weight_file=',w_fn];
qo12 = [' hardtarget_window_offset=','0'];
qo13 = [' log_file=',w_log,' log_weight_file=',w_chklog,' ckpt_weight_file=',w_chk];
qo14 = ' hardtarget_lastlab_reject=true';

if reject
cmdtxt = [qntrn,qo1,qo2,qo3,qo4,qo5,qo6,qo7,qo8,qo9,qo9a,qo10,qo11,qo12,qo13,qo14];
else
cmdtxt = [qntrn,qo1,qo2,qo3,qo4,qo5,qo6,qo7,qo8,qo9,qo9a,qo10,qo11,qo12,qo13];
end
disp(cmdtxt)
%%
system(cmdtxt);
