function [yf0,ap0,wf0,sf0,ss] = run_yws(filename,spos,epos,sad,dispflag)
% function [yf0,ap0,wf0,sf0,ss] = run_yws(filename,spos,epos,sad,dispflag)
%
% [2012-07-20] Byung Suk Lee bsl@ee.columbia.edu

addpath('../audioread/')
addpath('../yin/')
addpath('../SWIPEP/')
addpath('../wu/')

if ~exist('dispflag','var'), dispflag = 0; end;

freq_range = [60 2500];
force_mono = 1;
sr = 16000;
hop_in_s = 0.01;
P.hop = round(sr*hop_in_s);
wu_dispflag = 0;

[dum,name,ext] = fileparts(filename);
if exist('spos','var') && exist('epos','var')
[d,sr] = audioread(filename,sr,force_mono,spos,epos-spos);
else
[d,sr] = audioread(filename,sr,force_mono);
end
tmp_fn = [name,'_tmp.wav'];
wavwrite(d,sr,tmp_fn)

disp('::: Run YIN')
t_m(1) = cputime;
[r] = yin(tmp_fn,P);
ap0 = r.ap0;
yf0 = (440) .* (2 .^ (r.f0));
t_m(2)  = cputime;

disp('::: Run Wu')
[pw] = wu_main_rev(tmp_fn,wu_dispflag);
wf0 = period2freq(pw ./ 16);
t_m(3) = cputime;

disp('::: Run SWIPE''')
[sf0,tt,ss] = swipep(d,sr,freq_range, hop_in_s);
sf0 = sf0'; tt = tt'; ss = ss';
t_m(4) = cputime;

delete(tmp_fn);

t_d = diff(t_m);
disp(['YIN: ',num2str(t_d(1)),'s, Wu: ',num2str(t_d(2)),...
    's, SWIPE'': ',num2str(t_d(3)),'s'])

if dispflag
dspsec = length(d)/sr;
figure,
subplot(311)
plot((1:length(d))/sr,d)
set(gca,'XTickLabel',[]);
ylabel('audio')
xlim([0 dspsec])
subplot(312)
plot((1:length(yf0))*hop_in_s,yf0)
hold on;
plot((1:length(sf0))*hop_in_s,sf0,'r')
plot((1:length(wf0))*hop_in_s,wf0,'g')
xlim([0 dspsec])
set(gca,'XTickLabel',[]);
ylabel('freq / Hz')
ylim([0 2000])
legend('YIN','SWIPE''','Wu')
subplot(313)
xx = (1:length(r.ap0))*hop_in_s + spos;
plot(xx,r.ap0)
hold on;
xx = (1:length(ss))*hop_in_s + spos;
plot(xx,ss,'r')
xx = (1:length(sad))*hop_in_s;
plot(xx,sad,'y')
xlim([0 dspsec]+spos)
ylabel('strength')
xlabel('time / s')
legend('YIN','SWIPE''','SAD')
end

