function [yf0,ap0,wf0,sf0,ss] = run_yws_slice(filename,spos,epos,sad,dispflag)
% function [yf0,ap0,wf0,sf0,ss] = run_yws_slice(filename,spos,epos,sad,dispflag)
%
% [2012-07-20] Byung Suk Lee bsl@ee.columbia.edu

%%
addpath('../audioread/')
if ~exist('n_s','var'), n_s = 10; end;

sr = 16000;
force_mono = 1;

[dum,name,ext] = fileparts(filename);
if exist('spos','var') && exist('epos','var')
[d,sr] = audioread(filename,sr,force_mono,spos,epos-spos);
else
[d,sr] = audioread(filename,sr,force_mono);
end

%%
dlen = length(d);
buf_len = round(0.1*sr); % buffer length of front and end of slice
nslice = max(ceil(length(d) / (n_s * sr))-1,1);
nfrms = 1 + max(floor((size(d,1) - 0.025*sr)/(0.01*sr)),0);
yf0 = zeros(1,nfrms);
ap0 = zeros(1,nfrms);
wf0 = zeros(1,nfrms);
sf0 = zeros(1,nfrms);
ss = zeros(1,nfrms);
%% feature calculation on slices of length n_s sec
pos = 0;
lslice = 100 * n_s;
for islice = 1:nslice
    ids = n_s*sr*(islice-1) - buf_len * (islice > 1) + 1; % front buffer, except the first slice
    tide = min( n_s*sr*islice + buf_len, dlen);
    if islice == nslice
        ide = dlen;
    else
        ide = tide; % end buffer, always
    end
    disp(['(',num2str(islice),'/',num2str(nslice),') Start: ',num2str(ids),' End: ',num2str(ide)]);
    xx = d(ids:ide);
    tmp_fn = [name,num2str(islice),'.wav'];
    wavwrite(xx,sr,tmp_fn);
    [tyf0,tap0,twf0,tsf0,tss] = run_yws(tmp_fn);
    delete(tmp_fn);
    % cut frount and end buffer results
    tmpidxs = 10*(islice > 1);
    if islice < nslice
        idx = pos + (1:lslice);
        tmpidx = (tmpidxs + 1):(tmpidxs + lslice);
        yf0(idx) = tyf0(tmpidx);
        ap0(idx) = tap0(tmpidx);
        wf0(idx) = twf0(tmpidx);
        sf0(idx) = tsf0(tmpidx);
        ss(idx) = tss(tmpidx);
    else
        lslice = nfrms - (nslice - 1)*100*n_s;
        idx = pos + (1:lslice);
        tmpidx = (tmpidxs + 1):(tmpidxs + lslice);
        yf0(idx) = tyf0(tmpidx);
        ap0(idx) = tap0(tmpidx);
        wf0(idx) = twf0(tmpidx);
        sf0(idx) = tsf0(tmpidx);
        ss(idx) = tss(tmpidx);
    end
    disp(['islice: ',num2str(islice),' lslice: ',num2str(lslice)]);
    pos = pos + lslice;
end

